#include <iostream>
#include <queue>
#include <cstdio>
#include <cmath>
using namespace std;

const int MAXG = 100;
const double PI = 4.0*atan(1.0);
const double INF = 1e100;

class angleRange
{
public:
    double a1, a2;
    angleRange* next;

    angleRange(double aa1, double aa2, angleRange* n=0) : a1(aa1), a2(aa2), next(n)
    {}
};

class edge
{
public:
    double perc;
    int w;
    edge* next;

    edge(int ww, double p, edge* n=0) : w(ww), perc(p), next(n)
    {}
};

struct vertex
{
    angleRange* alist;
    edge* list;
    double x, y, z, r;
    double v;
    double fillRate;
    double currFill;
    double t;
    bool done;
} g[MAXG];

void printGraph(int n)
{
    for(int i=0; i<n; i++) {
        cout << i << ": " << g[i].v << ',' << g[i].fillRate << ',' << g[i].currFill << ',' << g[i].t << endl;
        for(edge* p = g[i].list; p!= 0; p = p->next)
            cout << " " << p->w << ',' << p->perc;
        cout << endl;
    }
}
double dist(double x1, double y1, double x2, double y2)
{
    double dx = x1-x2;
    double dy = y1-y2;
    return sqrt(dx*dx+dy*dy);
}

void circleInt(double cx1, double cy1, double r1, double cx2, double cy2, double r2, double& ix1, double& iy1, double& ix2, double& iy2)
{
    // expand (x-cx1)^2+(y-cy1)^2=r1^2 and the second circle equation, subtract to get a linear eqn in x and y, solve for y and put back into
    //   first circle equation and solve for x

    if (cy1 != cy2) {
        double p1 = (r1*r1-cx1*cx1-cy1*cy1 - (r2*r2-cx2*cx2-cy2*cy2))/2.0;
        double p2 = (cx1-cx2)/(cy2-cy1);
        double p3 = p1/(cy2-cy1);
        double a = 1 + p2*p2;
        double b = -2*cx1 + 2*p2*p3 - 2*p2*cy1;
        double c = cx1*cx1 + p3*p3 - 2*p3*cy1 + cy1*cy1 - r1*r1;
//cout << p1 << ' ' << p2 << ' ' << p3 << ' ' << a << ' ' << b << ' '<< c << endl;
        double disc = sqrt(b*b-4*a*c);
        ix1 = (-b+disc)/(2.0*a);
        iy1 = p2*ix1+p3;
        ix2 = (-b-disc)/(2.0*a);
        iy2 = p2*ix2+p3;
    }
    else {
        ix1 = ix2 = (r1*r1 - r2*r2- cx1*cx1 + cx2*cx2)/(cx2-cx1)/2.0;
        double disc = sqrt(r1*r1-(ix1-cx1)*(ix1-cx1));
        iy1 = cy1+disc;
        iy2 = cy1-disc;
    }
}

void circleInt(double cx1, double cy1, double r1, double cx2, double cy2, double r2, double& a1, double& a2)
{
    double ix1, iy1, ix2, iy2;

    circleInt(cx1, cy1, r1, cx2, cy2, r2, ix1, iy1, ix2, iy2);
/*
    cout << '(' << ix1 << ',' << iy1 << ')' << endl;
    cout << '(' << ix2 << ',' << iy2 << ')' << endl;
    cout << (ix1-cx1)*(ix1-cx1) + (iy1-cy1)*(iy1-cy1) << endl;
    cout << (ix1-cx2)*(ix1-cx2) + (iy1-cy2)*(iy1-cy2) << endl;
    cout << (ix2-cx1)*(ix2-cx1) + (iy2-cy1)*(iy2-cy1) << endl;
    cout << (ix2-cx2)*(ix2-cx2) + (iy2-cy2)*(iy2-cy2) << endl;
/**/
    a1 = atan2(iy1-cy1, ix1-cx1);
    a2 = atan2(iy2-cy1, ix2-cx1);
    if (a1 > a2)
        swap(a1, a2);
//    perc = (a2-a1)/(2*PI);
    double x = cx1-r1;
    double y = cy1-0.01;
    if ((x-cx2)*(x-cx2)+(y-cy2)*(y-cy2)-r2*r2 < 0.0) {    // check if -pi on circle 1 is inside circle 2
        swap(a1, a2);
//        perc = 1 - perc;
    }
}
double calcPerc(int c1, double a1, double a2)
{
//cout << "a1, a2 = " << a1 << ',' << a2 << endl;
    if (a2 < a1) {
        return calcPerc(c1, -PI, a2) + calcPerc(c1, a1, PI);
    }
    else {
        angleRange* p1 = g[c1].alist;
        bool inside1 = false;
        double astart = a1;
        while (p1->next != 0 && a1 > p1->next->a2)
            p1 = p1->next;
        if (p1->next != 0 && a1 >= p1->next->a1) {
            inside1 = true;
            astart = p1->next->a2;
        }
//cout << "astart = " << astart << endl;
        angleRange* p2 = p1->next;
        bool inside2 = true;
        double perc = 0.0;
        while (p2 != 0 && a2 > p2->a1) {
            if (p2->a1 > astart)            // come back to look at this
                perc += p2->a1 - astart;
            astart = p2->a2;
            p2 = p2->next;
        }
        if (a2 > astart) {
            inside2 = false;
            perc += a2 - astart;
//cout << "  added " << a2-astart << " to perc" << endl;
        }
//cout << "insides: " << inside1 << ',' << inside2 << endl;
//cout << "p1, p2 values = " << p1->a1 << ',' << p2->a1 << endl;
        if (!inside1)
            p1->next = new angleRange(a1,a2, p1->next);
        p1 = p1->next;
        while (p1->next != p2) {
            angleRange* tmp = p1->next;
            p1->a2 = p1->next->a2;
            p1->next = p1->next->next;
            delete tmp;
        }
        if (!inside2) {
            p1->a2 = a2;
        }
        return perc/(2*PI);
    }
}

void getNextFillTime(int n, double& t, int& v)
{
    t = INF;
    for(int i=0; i<n; i++) {
        if (!g[i].done && g[i].t < t) {
            t = g[i].t;
            v = i;
        }
    }
}

int main()
{
    int n;
    double a1, a2, perc;

    cin >> n;
    for(int i=0; i<n; i++) {
        vertex tmp;
        cin >> tmp.x >> tmp.y >> tmp.z >> tmp.r >> tmp.v;
        int j=i;
        while (j > 0 && g[j-1].z < tmp.z) {
            g[j] = g[j-1];
            j--;
        }
        g[j] = tmp;
        g[j].alist = new angleRange(-1, -1);
        g[j].list = 0;
        g[j].fillRate = g[i].currFill = 0.0;
        g[j].t = INF;
        g[j].done = false;
    }
    for(int i=0; i<n-1; i++) {
        for(int j=i+1; j<n; j++) {
            if (g[i].z <= g[j].z)
                continue;
            double d = dist(g[i].x, g[i].y, g[j].x, g[j].y);
            if (d >= g[i].r+g[j].r || d + g[j].r <= g[i].r)     // no intersect or lower contained in upper
                continue;
            if (d+g[i].r <= g[j].r) {                           // upper contained in lower
                a1 = -PI;
                a2 = PI;
            }
            else
                circleInt(g[i].x, g[i].y, g[i].r, g[j].x, g[j].y, g[j].r, a1, a2);
            perc = calcPerc(i, a1, a2);
            g[i].list = new edge(j, perc, g[i].list);
/*
cout << "i, j = " << i << ',' << j << ": perc = " << perc;
for(angleRange* p = g[i].alist; p!= 0; p=p->next)
cout << " (" << p->a1 << ',' << p->a2 << ")";
cout << endl;
/**/
        }
    }

    g[0].fillRate = 100.0;
    g[0].t = g[0].v/100.0;
//printGraph(n);
    double lastt = 0.0, currt;
    int v;
    for(int i=0; i<n; i++) {
        getNextFillTime(n, currt, v);
        if (currt == INF)
            break;
//cout << "currt, v = " << currt << ',' << v << endl;
        for(int j=0; j<n; j++) {
            if (!g[j].done) {
                g[j].currFill += (currt - lastt)*g[j].fillRate;
            }
        }
        g[v].done = true;
        queue<int> q;
        q.push(v);
        while (!q.empty()) {
            v = q.front();
            q.pop();
            for(edge* p = g[v].list; p!=0; p = p->next) {
                int w = p->w;
                if (g[w].done) {
                    g[w].fillRate = g[v].fillRate*p->perc;  // CHEAT: this is actually adding on to existing fillRate
                    q.push(w);
//cout << "enqueue: " << w << " from " << v << endl;
                }
                else {
                    g[w].fillRate += g[v].fillRate*p->perc;
                    g[w].t = currt + (g[w].v - g[w].currFill)/g[w].fillRate;
                }
            }
        }
        lastt = currt;
    }
    if (currt == INF)
        cout << "Invalid" << endl;
    else
    {
        printf("%.2f", currt);
        cout << endl;
    }
//        cout << currt << endl;
}
